import java.io.IOException;
import java.util.concurrent.TimeoutException;

import org.apache.commons.lang.math.NumberUtils;
import org.apache.log4j.Level;
import org.jetel.component.AbstractGenericTransform;
import org.jetel.data.DataRecord;
import org.jetel.exception.ComponentNotReadyException;
import org.jetel.exception.ConfigurationStatus;
import org.jetel.exception.ConfigurationStatus.Priority;
import org.jetel.exception.ConfigurationStatus.Severity;
import org.jetel.exception.JetelRuntimeException;
import org.jetel.metadata.DataFieldType;
import org.jetel.metadata.DataRecordMetadata;

import com.rabbitmq.client.Channel;
import com.rabbitmq.client.Connection;
import com.rabbitmq.client.ConnectionFactory;
import com.rabbitmq.client.GetResponse;
import com.rabbitmq.client.impl.recovery.AutorecoveringChannel;

public class RabbitMqReader extends AbstractGenericTransform {
	
	private static final String MESSAGE_ID_FIELD_NAME = "messageId";
	private static final String RMQ_CHANNEL_DICTIONARY = "rabbitMqChannel";
	private static final String TENANT_FIELD_NAME = "messageContent";
	private static final String MESSAGE_TIMESTAMP = "messageTimestamp";
	private static final String RMQ_CONNECTION_USERNAME = "RMQ_USERNAME";
	private static final String RMQ_CONNECTION_PASSWORD = "RMQ_PASSWORD";
	private static final String RMQ_CONNECTION_HOST = "RMQ_HOST";
	private static final String RMQ_CONNECTION_PORT = "RMQ_PORT";
	private static final String RMQ_CONNECTION_QUEUE = "RMQ_QUEUE";
	
	/**
	 *  RabbitMQ connection, this component creates connection that is further used in other components (e.g. for calling RMQ basicAck)
	 */
	private Connection conn;
	
	/**
	 *  RabbitMQ channel that is opened in this component and distributed through dictionary to other components
	 */
	private Channel channel;
	
	@Override
	public void execute() {
		
		// Retrieving messages from a queue
		getLogger().log(Level.INFO, "Pulling data from the queue '" + RMQ_CONNECTION_QUEUE + "'...");
		boolean autoAck = false;
		
		try {
			GetResponse response = channel.basicGet(graph.getGraphParameters().getGraphParameter(RMQ_CONNECTION_QUEUE).getValue(), autoAck);
			DataRecord record = outRecords[0];
			int i = 1;
			while(response != null) {
				record.getField(TENANT_FIELD_NAME).setValue(response.getBody());
				record.getField(MESSAGE_ID_FIELD_NAME).setValue(i++);
				record.getField(MESSAGE_TIMESTAMP).setValue(response.getProps().getTimestamp());
				writeRecordToPort(0, record);
				record.reset();
				
				response = channel.basicGet(graph.getGraphParameters().getGraphParameter(RMQ_CONNECTION_QUEUE).getValue(), autoAck);
			}
			
		} catch (IOException e) {
			throw new JetelRuntimeException("Communication with RabbitMQ failed.", e);
		}
		
	}

	@Override
	public ConfigurationStatus checkConfig(ConfigurationStatus status) {
		super.checkConfig(status);
		
		return status;
	}

	@Override
	public void init() {
		super.init();	
		
	}

	@Override
	public void preExecute() throws ComponentNotReadyException {
		super.preExecute();
		
		// Starting with the RMQ connection
		getLogger().log(Level.INFO, "Connecting to RabbitMQ...");
		ConnectionFactory factory = new ConnectionFactory();
		factory.setUsername(graph.getGraphParameters().getGraphParameter(RMQ_CONNECTION_USERNAME).getValue());
		factory.setPassword(graph.getGraphParameters().getGraphParameter(RMQ_CONNECTION_PASSWORD).getValue());
		factory.setVirtualHost(graph.getGraphParameters().getGraphParameter(RMQ_CONNECTION_HOST).getValue());
		factory.setPort(Integer.parseInt(graph.getGraphParameters().getGraphParameter(RMQ_CONNECTION_PORT).getValue()));	
	
		
		try {
			// Connect to RMQ
			conn = factory.newConnection();
			////conn = factory.newConnection();
			getLogger().log(Level.DEBUG, "Successfully connected to RabbitMQ...");
			
			// Connection created, creating channel
			getLogger().log(Level.INFO, "Creating channel...");
			channel = conn.createChannel();
			getLogger().log(Level.DEBUG, "Successfully created new channel...");
			
			// Insert channel into dictionary
			graph.getDictionary().setValue(RMQ_CHANNEL_DICTIONARY, channel);
			getLogger().log(Level.DEBUG, "Channel successfully assigned to dictionary entry '" + RMQ_CHANNEL_DICTIONARY + "'...");
					
		} catch (IOException e) {
			throw new JetelRuntimeException("Communication with RabbitMQ failed.", e);
		} catch (TimeoutException e) {
			throw new JetelRuntimeException("Connection to RabbitMQ timmed out!", e);
		} catch (ComponentNotReadyException e) {
			throw new JetelRuntimeException("Failed to assign channel to a dictionary!", e);
		}	
	}

	@Override
	public void postExecute() throws ComponentNotReadyException {
		super.postExecute();

		try {
			channel.close();
			conn.close();
		} catch (IOException | TimeoutException e) {
			getLogger().log(Level.WARN, "Problems with closing connection/channel occured...");
		}
		
	}
	
	@Override
	public void free() {
		// TODO Auto-generated method stub
		super.free();
	}
	
}
